"""
WSGI config for demo project.

This module contains the WSGI application used by Django's development server
and any production WSGI deployments. It should expose a module-level variable
named ``application``. Django's ``runserver`` and ``runfcgi`` commands discover
this application via the ``WSGI_APPLICATION`` setting.

Usually you will have the standard Django WSGI application here, but it also
might make sense to replace the whole Django WSGI application with a custom one
that later delegates to the Django one. For example, you could introduce WSGI
middleware here, or combine a Django application with an application of another
framework.

"""
import os
import sys
import site

os.environ['DJANGO_SETTINGS_MODULE'] = 'demo.settings'

SITE_ROOT = os.path.dirname(os.path.dirname( __file__ ))

site.addsitedir( SITE_ROOT + '/venv/local/lib/python2.7/site-packages' )

sys.path.append( SITE_ROOT )

exc_dir = 'scripts' if os.name == 'nt' else 'bin'
venv = '%s/venv/%s/activate_this.py' % (SITE_ROOT, exc_dir )

activate_env = os.path.expanduser( venv )
execfile( activate_env, dict(__file__ = activate_env ))

import django.core.handlers.wsgi
application = django.core.handlers.wsgi.WSGIHandler()



import json
from django.http import HttpResponse


def upload_receive( request ):
    """
    Returns the file(s) uploaded by the user.
    """
    return request.FILES['files[]'] if request.FILES else None


class JFUResponse( HttpResponse ):
    """
    Returns an HTTP response with its data encoded in JSON format.

    Content-Type negotation is handled transparently by inspecting the
    request object. If the client accepts JSON, the MIME-type of the response
    is set to JSON. Otherwise, the MIME-type defaults to 'text/plain'.

    This approach suports jQuery File Upload's use of an Iframe Transport
    module for the browsers (IE & Opera) incapable of handling XmlHttpRequest
    file uploads and who present download dialogs for iFrame responses that
    declare a JSON MIME-type.
    """

    def __init__( self, request, data = True, *args, **kwargs ):
        data   = json.dumps( data )
        j      = "application/json"
        accept = request.META.get('HTTP_ACCEPT')

        mime = j if accept and j in accept else 'text/plain'

        super( JFUResponse, self ).__init__( data, mime, *args, **kwargs )
    

class UploadResponse( JFUResponse ):
    """
    Takes a dictionary containing the required jQuery File Upload fileupload
    response data and returns a JFUResponse.

    The dictionary may take the following form:
    {
       'name': 'file name', 
       'size': 12345,
       'url' : 'file url', 
       'thumbnailUrl': 'thumbnail url',
       'deleteUrl'   : 'delete url',
       'deleteType'  : 'delete type', 
    }

    Refer to the jQuery File Upload documentation for acceptable response data.
    [https://github.com/blueimp/jQuery-File-Upload/wiki/Setup]
    """

    def __init__( self, request, file_dict, *args, **kwargs ):
        files = file_dict if isinstance( file_dict, list ) else [ file_dict ]
        data  = { 'files' : files }
        super( UploadResponse, self ).__init__( request, data, *args, **kwargs )



from django.conf import settings
from django.conf.urls import patterns, include, url
from django.conf.urls.static import static

from photos import views

urlpatterns = patterns('',
    url( r'^$', views.Home.as_view(), name = 'home' ),

    url( r'upload/', views.upload, name = 'jfu_upload' ),
    url( r'^delete/(?P<pk>\d+)$', views.upload_delete, name = 'jfu_delete' ),
)

urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT) 



import os
from django.conf import settings
from django.core.urlresolvers import reverse
from django.views import generic
from django.views.decorators.http import require_POST
from jfu.http import upload_receive, UploadResponse, JFUResponse
from photos.models import Photo

class Home( generic.TemplateView ):
    template_name = 'base.html'

    def get_context_data(self, **kwargs):
        context = super( Home, self ).get_context_data( **kwargs )
        context['accepted_mime_types'] = ['image/*']
        return context

@require_POST
def upload( request ):
    file = upload_receive( request )

    instance = Photo( file = file )
    instance.save()

    basename = os.path.basename( instance.file.path )
    
    file_dict = {
        'name' : basename,
        'size' : file.size,

        'url': settings.MEDIA_URL + basename,
        'thumbnailUrl': settings.MEDIA_URL + basename,

        'deleteUrl': reverse('jfu_delete', kwargs = { 'pk': instance.pk }),
        'deleteType': 'POST',
    }

    return UploadResponse( request, file_dict )

@require_POST
def upload_delete( request, pk ):
    success = True
    try:
        instance = Photo.objects.get( pk = pk )
        os.unlink( instance.file.path )
        instance.delete()
    except Photo.DoesNotExist:
        success = False

    return JFUResponse( request, success )



from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.template import Library, Context, loader

register = Library()

@register.simple_tag( takes_context = True )
def jfu(
        context,
        template_name = 'jfu/upload_form.html',
        upload_handler_name = 'jfu_upload',
        *args, **kwargs
    ):
    """
    Displays a form for uploading files using jQuery File Upload.

    A user may use both a custom template or a custom upload-handling URL
    name by supplying values for template_name and upload_handler_name
    respectively.

    Any additionally supplied positional and keyword arguments are directly
    forwarded to the named custom upload-handling URL.
    """
    context.update( { 
        'JQ_OPEN'  : '{%',
        'JQ_CLOSE' : '%}',
        'upload_handler_url': reverse(
            upload_handler_name, args = args, kwargs = kwargs
        ),
    } )

    # Use the request context variable, injected
    # by django.core.context_processors.request,
    # to generate the CSRF token.
    context.update( csrf( context.get('request') ) )

    t = loader.get_template( template_name )

    return t.render( Context( context ) )
